"""The main code with all the modules loading and running them. Will have to adapt to the new code 927
The final step would be to add the sampling algorithm here...."""
import os.path

from data_loader import list_data, list_directories, list_files, loadx_variable
from series_extractor import TimeSeriesExtractor
from arima_model import plot_time_series
from lstm_model import LSTMModel
from utils import load_config
from utils_plot import Plotter
from utils_plot import plot_correlations
import time
from datetime import datetime




def main():



    # Loads the configuration file with all settings
    config = load_config('config.yaml')

    # Lists the main directory of the data
    path = config['data_path']
    version = config['version']
    index_pairs = config['index_pairs']
    data = list_data(path)
    files = list_files(path)
    dir = list_directories(path)
    check_variable = config['variables']

    # Access time sequence parameters
    train_start_year = config['time_sequence']['train_start_year']
    train_end_year = config['time_sequence']['train_end_year']
    test_start_year = config['time_sequence']['test_start_year']
    test_end_year = config['time_sequence']['test_end_year']

    # Access datasets, variables, and scaling factors
    data_path = config['data_path']
    version = config['version']
    index_pairs = config['index_pairs']
    area_dict= config['area']
    dictionary = config['variablex']
    data = config['data']
    k_factors = config['k_factors']
    output_dir = config["output"]
    output_csv = os.path.join(output_dir,config["cross_corr"])
    output_lstm = os.path.join(output_dir,config["lstm_model"]['output_lstm'])

    # Create a dictionary to hold the datasets
    datasets = {}

    # Load data for each variable

    datasets = loadx_variable(path,dir,check_variable,version)

    #print("~"*100)
    #print("main() ai4pisces")
    #print("dictionary.items()",dictionary.items())
    #print("~"*100)
    # Initialize the TimeSeriesExtractor with the loaded datasets
    extractor = TimeSeriesExtractor(data,dictionary,datasets, k_factors, train_start_year, train_end_year, test_start_year, test_end_year)



    # Extract and process time series data for the given index pairs
    time_series_data = extractor.get_time_series_for_multiple_points(index_pairs)

    # Convert the processed time series data into dataframes
    df_train, df_test = extractor.convert_to_dataframe(time_series_data)




    for area in area_dict:
        extractor.add_lagged_series(df_train, df_test, 'intpp', area)


    print("df_train.head() ",df_train.head())
    print("df_test.head() ",df_test.head())

    # After calculating cross-correlations
    #cross_corr_df = extractor.calculate_cross_correlations(time_series_data, max_lag=3)

    # Save the DataFrame to a CSV file
    print("output_csv",output_csv)

    #cross_corr_df.to_csv(output_csv, index=False)

    # Plot pairwise cross-correlations
    #plot_correlations(cross_corr_df)


    # Get common latitudes and longitudes (assuming all datasets have the same)
    #print("datasets.values()...",datasets.values())
    #print("iter(datasets.values())....",iter(datasets.values()))
    #print("next(iter(datasets.values()))...",next(iter(datasets.values())))
    sample_dataset = next(iter(datasets.values()))
    latitudes = sample_dataset['latitude']
    longitudes = sample_dataset['longitude']

    # Iterate over all variables and index pairs to plot the series as subplots
    #for variable, dataset in datasets.items():
        # Initialize the Plotter class
    #    plotter = Plotter(
    #        data_array=dataset,
    #        latitudes=latitudes,
    #        longitudes=longitudes,
    #        datasets=datasets,
    #        output_dir='plots'
    #    )

        # Call the plot_series method for each variable
    #    plotter.plot_series(index_pairs)

    # Initiate the Plotter class to the plot_with_markers method for each variable
    #plotter = Plotter(
    #        data_array=sample_dataset,
    #        latitudes=latitudes,
    #        longitudes=longitudes,
    #       datasets=datasets,
    #         output_dir='plots'
    #    )

    #time_index = 100

    #plotter.plot_with_markers(time_index, index_pairs)


    #scaled_data, scaler = preprocess_data(data)

    # ARIMA Model
    #arima_model = train_arima_model(scaled_data, **config['arima_params'])
    #plot_arima_results(scaled_data, arima_model, 10, config['output_path'])

    # LSTM Model
    print("output_lstm ",output_lstm)
    lstm = LSTMModel(look_back=10, epochs=100, batch_size=32, output_lstm=output_lstm)
    lstm.run_lstm(df_train, df_test, area_dict=config['area'],
                  normx_dict=config['normx_dict'], columns=config['columns'],
                  variables=config['variables'], n_forecast_steps=config['n_forecast_steps'])



if __name__ == "__main__":
    start_time_main = time.time()
    print("Running main.............",datetime.now().strftime("%Y-%m-%d_%HH:%MM:%SS"))
    main()
    end_time_main = time.time()
    time_used_main = (end_time_main - start_time_main)
    print("Time used for entire Code.../zzzz", f'{time_used_main:.2f}', "Seconds")
    print("Time used for entire Code..../zzzz", f'{time_used_main/60:.2f}', "Minutes")
    print("Time used for entire Code..../zzzz", f'{time_used_main/3600:.2f}', "Hours")