import numpy as np
import pandas as pd
import time
import scipy



class TimeSeriesExtractor:
    def __init__(self, data,dictionary,datasets, k_factors, train_start_year, train_end_year, test_start_year, test_end_year):
        self.datasets = datasets  # A dictionary of datasets
        self.k_factors = k_factors  # Dictionary of scaling factors for each variable
        self.train_start_year = train_start_year
        self.train_end_year = train_end_year
        self.test_start_year = test_start_year
        self.test_end_year = test_end_year
        self.dictionary = dictionary
        self.data = data

    def extract_time_series(self, var_name, i_index, j_index, lev=None, method='nearest'):
        print("lev", lev)
        if lev is not None:
            lev_end = lev
            lev_start = 0
            time_series = self.datasets.get(var_name.upper()).get(var_name).isel(i=i_index, j=j_index, lev=slice(lev_start, lev_end + 1)).mean(dim='lev')
            #time_series = self.datasets[self.data][var_name].isel(i=i_index, j=j_index, lev=slice(lev_start, lev_end + 1)).mean(dim='lev')
        else:
            print("~"*100)
            print("def extract_time_series(self, var_name, i_index, j_index, lev=None, method='nearest')")
            #print("self.data",self.data)
            #print("var_name",var_name)

            #print("self.data[var_name]",self.data[var_name])
            #print("f{self.data[var_name]}",f"{self.data[var_name]}")
            #print("type({self.data[var_name]})",type(f"{self.data[var_name]}"))

            #print("self.datasets",self.datasets)

            # Access the dictionary for the variable dynamically
            #print("self.datasets.get(var_name)", self.datasets.get(var_name.upper()))
            #print("getattr(self.datasets, var_name)", getattr(self.datasets, var_name.upper()))

            #print("self.datasets.get(var_name).get(var_name)", self.datasets.get(var_name.upper()).get(var_name))

            #print("self.datasets[{self.data[var_name]}]",self.datasets[f"{self.data[var_name].upper()}"])
            #self.datasets.get(var_name)
            #y = f"{self.data[var_name].upper()}"
            #print("self.datasets.[y]", self.datasets[y])
            #print("self.datasets.[y].x", self.datasets[y].x)





            #time_series = self.datasets[self.data[var_name]][var_name].isel(i=i_index, j=j_index)
            time_series = self.datasets.get(var_name.upper()).get(var_name).isel(i=i_index, j=j_index)
            #print("self.datasets.get(var_name.upper()).get(var_name)",self.datasets.get(var_name.upper()).get(var_name))
            #print("self.datasets.get(var_name.upper()).get(var_name).isel(i=i_index, j=j_index)",self.datasets.get(var_name.upper()).get(var_name).isel(i=i_index, j=j_index))

            #print("time_series.values",time_series.values)
            #print("time_series",time_series)
            #print("time_series[var_name]",time_series[var_name])
            print("~" * 100)
        return time_series#[var_name]


    def normalize_time_series(self, time_series):
        mean = np.mean(time_series)
        std = np.std(time_series)
        normalized_series = (time_series - mean) / (std + 0.01)
        return normalized_series

    def k_scaled_timeseries(self, time_series, k_factor):
        print("~" * 100)
        print("def k_scaled_timeseries(self, time_series, k_factor):")
        print("self.k_factors ",self.k_factors)
        print("k_factor ",k_factor)
        print("type(k_factor) ",type(k_factor))
        print("~" * 100)
        k_scaled_series = np.log1p(np.abs(time_series) * float(k_factor) + np.exp(2)) + 12

        return k_scaled_series

    def replace_nan_with_zero(self, time_series):
        print("~" * 100)
        print("def replace_nan_with_zero(self, time_series)::")
        print("self.k_factors ",self.k_factors)
        time_series[np.isnan(time_series)] = 0
        return time_series

    def get_time_series_for_multiple_points(self, index_pairs):
        print("~" * 100)
        print("get_time_series_for_multiple_points(self, index_pairs):")
        all_time_series_data = {}
        for (i_index, j_index) in index_pairs:
            time_series_data = {}

            for key, variable in self.dictionary.items():

                print("for key, variable in self.dictionary.items():")
                print("key", key)
                print("variable['name']",variable['name'])

                lev = variable.get('lev', None)
                print("level",lev)
                print("self.extract_time_series(variable['name'])")
                #print("self.extract_time_series(variable['name'])",self.extract_time_series(variable['name'],120,290,lev))
                time_series = self.extract_time_series(variable['name'], i_index, j_index, lev).values
                time_series = self.replace_nan_with_zero(time_series)
                normalized_series = self.normalize_time_series(time_series)
                k_factor = self.k_factors.get(key, 1)
                k_scaled_series = self.k_scaled_timeseries(time_series, k_factor)
                normalized_series = self.replace_nan_with_zero(normalized_series)
                k_scaled_series = self.replace_nan_with_zero(k_scaled_series)

                train_mask = (self.datasets.get(key.upper()).get(key).time.dt.year >= self.train_start_year) & (self.datasets.get(key.upper()).get(key).time.dt.year <= self.train_end_year)
                test_mask = (self.datasets.get(key.upper()).get(key).time.dt.year >= self.test_start_year) & (self.datasets.get(key.upper()).get(key).time.dt.year <= self.test_end_year)

                time_series_data[key] = {
                    'normalized_train': normalized_series[train_mask],
                    'normalized_test': normalized_series[test_mask],
                    'k_scaled_train': k_scaled_series[train_mask],
                    'k_scaled_test': k_scaled_series[test_mask]
                }
            all_time_series_data[(i_index, j_index)] = time_series_data
        print("~"*100)
        return all_time_series_data


    def time_function_execution(self, func, *args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        elapsed_time = end_time - start_time
        return result, elapsed_time

    def convert_to_dataframe(self, time_series_data):
        print("def convert_to_dataframe(self, time_series_data):")
        df_train = pd.DataFrame()
        df_test = pd.DataFrame()
        for point_key, series_dict in time_series_data.items():
            for variable_key, series_data in series_dict.items():
                train_col_names = {
                    'normalized': f"{variable_key}_normalized_train_{point_key}",
                    'k_scaled': f"{variable_key}_k_scaled_train_{point_key}"
                }
                test_col_names = {
                    'normalized': f"{variable_key}_normalized_test_{point_key}",
                    'k_scaled': f"{variable_key}_k_scaled_test_{point_key}"
                }

                df_train[train_col_names['normalized']] = series_data['normalized_train']
                df_test[test_col_names['normalized']] = series_data['normalized_test']
                df_train[train_col_names['k_scaled']] = series_data['k_scaled_train']
                df_test[test_col_names['k_scaled']] = series_data['k_scaled_test']
        print("variable_key: ",variable_key)
        time_train_index = self.datasets.get(variable_key.upper()).get(variable_key).time.sel(time=(self.datasets.get(variable_key.upper()).get(variable_key).time.dt.year >= self.train_start_year) & (self.datasets.get(variable_key.upper()).get(variable_key).time.dt.year <= self.train_end_year))
        time_test_index = self.datasets.get(variable_key.upper()).get(variable_key).time.sel(time=(self.datasets.get(variable_key.upper()).get(variable_key).time.dt.year >= self.test_start_year) & (self.datasets.get(variable_key.upper()).get(variable_key).time.dt.year <= self.test_end_year))

        df_train.index = time_train_index
        df_test.index = time_test_index

        return df_train, df_test

    def add_lagged_series(self, df_train, df_test, variable, point):
        point_key = point
        variable_key = variable

        train_col = {
            'normalized': f"{variable_key}_normalized_train_{point_key}",
            'k_scaled': f"{variable_key}_k_scaled_train_{point_key}"
        }
        test_col = {
            'normalized': f"{variable_key}_normalized_test_{point_key}",
            'k_scaled': f"{variable_key}_k_scaled_test_{point_key}"
        }

        lagged_column_train = {
            'normalized': f"{variable_key}_lag1_normalized_train_{point_key}",
            'k_scaled': f"{variable_key}_lag1_k_scaled_train_{point_key}"
        }
        lagged_column_test = {
            'normalized': f"{variable_key}_lag1_normalized_test_{point_key}",
            'k_scaled': f"{variable_key}_lag1_k_scaled_test_{point_key}"
        }

        df_train[lagged_column_train['normalized']] = df_train[train_col['normalized']].shift(1)
        df_test[lagged_column_test['normalized']] = df_test[test_col['normalized']].shift(1)
        df_train[lagged_column_train['k_scaled']] = df_train[train_col['k_scaled']].shift(1)
        df_test[lagged_column_test['k_scaled']] = df_test[test_col['k_scaled']].shift(1)

        df_train[lagged_column_train['normalized']].iloc[0] = np.nan  # First value is NaN
        df_test[lagged_column_test['normalized']].iloc[0] = df_train[lagged_column_train['normalized']].iloc[-1]  # Initial test value is the last train value
        df_train[lagged_column_train['k_scaled']].iloc[0] = np.nan  # First value is NaN
        df_test[lagged_column_test['k_scaled']].iloc[0] = df_train[lagged_column_train['k_scaled']].iloc[-1]  # Initial test value is the last train value


    def apply_cross_correlation(self, series1, series2, max_lag=3):
        series1 = np.array(series1)
        series2 = np.array(series2)
        cross_corr = {}

        for lag in range(-max_lag, max_lag + 1):
            if lag < 0:
                lagged_series1 = series1[:lag]
                lagged_series2 = series2[-lag:]
            elif lag > 0:
                lagged_series1 = series1[lag:]
                lagged_series2 = series2[:-lag]
            else:
                lagged_series1 = series1
                lagged_series2 = series2

            corr = scipy.stats.pearsonr(lagged_series1,lagged_series2)
            cross_corr[lag] = corr.statistic

        return cross_corr

    def calculate_cross_correlations(self, time_series_data, max_lag=3):
        results = []

        for point_key, series_dict in time_series_data.items():
            for var1_key, series1_data in series_dict.items():
                for var2_key, series2_data in series_dict.items():
                    if var1_key != var2_key:  # Avoid self-correlation
                        cross_corr = self.apply_cross_correlation(series1_data['normalized_train'], series2_data['normalized_train'], max_lag)
                        #print("cross_corr: ",cross_corr)
                        #print("cross_corr.items()",cross_corr.items())
                        for lag, corr_value in cross_corr.items():
                            results.append({
                                'point': point_key,
                                'var1': var1_key,
                                'var2': var2_key,
                                'lag': lag,
                                'correlation': corr_value
                            })

        return pd.DataFrame(results)


